//Datei: BACKCNT.C
// Hauptprogramm
// Ein/Aus Qultimer mit PIC16F84
//
// Fr zweizeilige LCD Anzeigen 2x8,2x16,...
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 10.05.2001
// Compiler CC5x

// Achtung:	int ist 8 Bit lang
//		char ist 8 Bit lang
//		long ist 16 Bit lang

// Der Takt des PIC betrgt 3.2768MHz
// Ein Maschinenzyklus damit 1.2207us

#include "prozess.h"      /* Der verwendete Prozessor */

#include "backcnt.h"
#include "protos.h"

#include "interupt.c"
#include "lcd.c"
#include "delay.c"
#include "binbcd16.c"
#include "eeprom.c"
#include "initpic.c"

 // Watchdog off, Power up Timer on, XT Oscillator, Code Protect off
#pragma config |= 0x3FF1 //Configuration Word

//Globale Flags und Rckgabevariablen
unsigned char stelle; 	//Position auf der LCD-Anzeige
unsigned char key;	//Zum Lesen der Tastatur
unsigned char result;	//Zwischenergebnisse
unsigned char tastbuf;  //Tastaturpuffer fr lesen im INT
unsigned char boing;    //Warten auf INT
unsigned char rep;      //Mehrfachauslsen vermeiden
unsigned long countdown; //16Bit Rckwrtszhler
unsigned long ontime;    //Einschaltzeit
unsigned long offtime;   //Ausschaltzeit
unsigned char mode;      //mode 0 => Aus, 1 => An
unsigned char gogo;      //Countdown an/aus
unsigned char lastmode;  //letzter bearbeiteter Mode

/* Hauptprogramm */
void main()
{
 InitPIC(); //Interrupts und so weiter
 gogo=DISABLE; //Kein Countdown

 LCDInit(); //LCD Anzeige einstellen

 LCDCls();  //Anzeige lschen
 LCDWrite("Stress");
 LCDPos(2,1);
 LCDWrite("Timer");
 Delay1ms(255);          //Anzeige etwas verlngern
 Delay1ms(255);          //Anzeige etwas verlngern
 Delay1ms(255);          //Anzeige etwas verlngern
 LCDCls();  //Anzeige lschen
 LCDPos(1,7); LCDWrite("On");
 LCDPos(2,7); LCDWrite("Of");
 
// Hier die gespeicherten Zeiten aus dem EEPROM lesen !!
 result=Read_EEProm(1);     //Highbyte
 ontime=(unsigned long)result;
 ontime<<=8;
 ontime+=Read_EEProm(0); //Lowbyte

 result=Read_EEProm(3);     //Highbyte
 offtime=(unsigned long)result;
 offtime<<=8;
 offtime+=Read_EEProm(2); //Lowbyte
 
 stelle=Read_EEProm(4);                //Cursorposition
 if((stelle>5) || (stelle==0)) stelle=1; //Position 1 als Default

 mode=OFF;
 ShowDisplay(); //Ausschaltzeit anzeigen
 mode=ON;
 ShowDisplay(); //Einschaltzeit anzeigen
 lastmode=mode;
 
 boing=0; //Flag "TIMERINT war da", sonst wird in der Schleife
 	  //key mehrfach bearbeitet !
 rep=0;   //Merkt sich den letzen Wert von boing
 	  
 while(1) // Anfang Programm Endlosschleife
  {

   if(boing>0) //Taste gedrckt ?
    {
     if(rep!=boing) //War ein neuer INT da ?
      {
       rep=boing;     //nchstes Auslsen erst wieder wenn neuer INT da war

       //Einzelauslsung
       if(boing==2) key=tastbuf; //Tastaturpuffer lesen, Lowaktiv

       //Ab hier schnelles Tastaturrepeat
       if(boing>20 && (boing&0x07)==0) key=tastbuf;  
      }
    }
   else rep=0; //Taste losgelassen, Tastenwiederholung lschen
   
   if(key==ONOFF)
    {
     gogo=DISABLE; //kein Countdown mehr
     RELAIS_OFF(); //Relais ausschalten

     mode=ON;      //Ein/Auszeiten wieder anzeigen
     ShowDisplay();
     mode=OFF;      
     ShowDisplay();

     mode=lastmode; //Zuletzt bearbeitete Zeit aktivieren
     ShowDisplay();
     
    }//if key==ONOFF
   
   if(gogo==DISABLE) //Folgende Tasten nur abfragen wenn kein Countdown
    {
     if(key==RIGHT)
      {
       if(stelle<6) stelle=stelle+1; // Eins nach rechts
       if(stelle==6)
        {
         stelle=1; // Ans linke Ende des Displays bei berlauf
         if(mode==ON) { mode=OFF; lastmode=mode; } //und Zeile wechseln
         else         { mode=ON; lastmode=mode; }
        } 
       ShowDisplay(); 		//Display updaten
      } 

     if(key==UP)
      {
       if(stelle==5 && countdown<65535) countdown++;      //eins hochzhlen
       if(stelle==4 && countdown<=65525) countdown+=10;    //zehn hochzhlen
       if(stelle==3 && countdown<=65435) countdown+=100;   //hundert hochzhlen
       if(stelle==2 && countdown<=64535) countdown+=1000;  //tausend hochzhlen
       if(stelle==1 && countdown<=55535) countdown+=10000; //zehntausend hochzhlen

       if(mode==ON) ontime=countdown;
       else offtime=countdown;

       ShowDisplay(); 		//Display updaten
      }

     if(key==DOWN)
      {
       if(stelle==5 && countdown>0) countdown--;      //eins runterzhlen
       if(stelle==4 && countdown>=10) countdown-=10;    //zehn runterzhlen
       if(stelle==3 && countdown>=100) countdown-=100;   //hundert runterzhlen
       if(stelle==2 && countdown>=1000) countdown-=1000;  //tausend runterzhlen
       if(stelle==1 && countdown>=10000) countdown-=10000; //zehntausend runterzhlen

       if(mode==ON) ontime=countdown;
       else offtime=countdown;
       
       ShowDisplay(); 		//Display updaten
      }

  //Hier STORE Taste auswerten und EEPROM beschreiben
     if(key==STORE)
      {
       LCDPos(1,6);
       LCDWrite("EEP"); //Speichervorgang anzeigen
       Delay1ms(255);
       result=(unsigned char)ontime; //Einschaltzeit Lowbyte
       Write_EEProm(0,result);
       result=(unsigned char)(ontime>>8); //Einschaltzeit Highbyte
       Write_EEProm(1,result);
       result=(unsigned char)offtime; //Ausschaltzeit Lowbyte
       Write_EEProm(2,result);
       result=(unsigned char)(offtime>>8); //Ausschaltzeit Highbyte
       Write_EEProm(3,result);
       Write_EEProm(4,stelle); //Letzte Cursorposition merken
       LCDPos(1,6);
       LCDWrite(" On"); //Store wieder lschen
       ShowDisplay();
      }

  //Hier START Taste auswerten und Countdown starten
     if(key==START && countdown>0)
      {
       TMR0=0;  //Wer wei wo der gerade stand
       mode=ON;      //Relais zuerst einschalten
       countdown=ontime;
       RELAIS_ON();  //Relais einschalten
       gogo=ENABLE;  //Countdown auslsen
      }

     key=0;
    } //if gogo==DISABLE
    
   if(gogo<=TICKS) //Countdown luft
    {
     ShowDisplay();
     Delay1ms(24);  //Sonst flackert der Cursor
    }

   if(gogo==ENDCOUNTDOWN) //aktueller Countdown beendet ?
    {
     if(mode==ON)
      {
       mode=OFF;          //mode auf aus wechseln
       RELAIS_OFF();      
       countdown=offtime; //Countdown neu setzen
      }
     else
      {
       mode=ON;          //mode auf an wechseln
       RELAIS_ON();      
       countdown=ontime; //Countdown neu setzen
      } 

     gogo=ENABLE;  //Countdown neu auslsen
    }
    
  } // Ende Programm Endlosschleife
}

//************************************************
// Zeigt die 5 Dezimalstellen auf dem LCD Display 
//************************************************
void ShowDisplay()
{
 if(mode==ON) LCDPos(1,1);
 else         LCDPos(2,1);
 
 if(gogo==DISABLE) //Ein/Auszeiten nur anzeigen wenn kein Countdown
  {                //Sonst Countdown selbst anzeigen
   if(mode==ON) countdown=ontime;
   else         countdown=offtime;
  }
  
//Integer zu BCD und anzeigen
 B0=(unsigned char)countdown;      //Lowbyte
 B1=(unsigned char)(countdown>>8); //Highbyte
 BinToBCD16(); //Binr wandeln in BCD Bytes

 //Der Maximalwert ist 65535
 LCDWriteByte(R0+0x30);  //Nur das Low-Nibble des oberen Bytes anzeigen
 ShowBCD(R1);                   //tausender hunderter
 ShowBCD(R2);                   //zehner einer

if(mode==ON) LCDPos(1,stelle); // Cursor auf alte Dezimalstelle
else LCDPos(2,stelle); 
}
